//--------------------------------------------------------------------
//	
//	TToolTipControls.cpp
//
//	Sub-classes the following BControls:
//		BButton
//		BCheckBox
//		BColorControl
//		BRadioButton
//		BPictureButton
//		BSlider
//		BTextControl
//
//  Written by Robert Polic
//
//--------------------------------------------------------------------

#include <stdlib.h>
#include <string.h>

#include <Application.h>

#include "TToolTipControls.h"
#include "TToolTip.h"


//====================================================================
//
// BButton sub-class that adds a ToolTip
//
//====================================================================

TToolTipButton::TToolTipButton(BRect rect,
							   const char *name,
							   const char *label,
							   BMessage *msg,
							   const char *tip,
							   uint32 resize_mask,
							   uint32 flags)
			   :BButton(rect, name, label, msg, resize_mask, flags),
			    fIn(false)
{
	// cache the ToolTip string
	fTip = (char *)malloc(strlen(tip) + 1);
	strcpy(fTip, tip);
}

//--------------------------------------------------------------------

TToolTipButton::~TToolTipButton()
{
	// kill off any current ToolTip
	if (fIn)
		be_app->PostMessage(eToolTipStop);
	// free cached ToolTip
	free(fTip);
}

//--------------------------------------------------------------------

void TToolTipButton::MouseMoved(BPoint where, uint32 code, const BMessage *msg)
{
	// if mouse has moved into our view, our window is active and it previously
	// wasn't in, send a message to start the ToolTip
	if ((Bounds().Contains(where)) && (Window()->IsActive())) {
		if (!fIn) {
			BMessage	msg(eToolTipStart);

			msg.AddPoint("start", ConvertToScreen(where));
			msg.AddRect("bounds", ConvertToScreen(Bounds()));
			msg.AddString("string", fTip);
			be_app->PostMessage(&msg);
			fIn = true;
		}
	}
	// otherwise stop the message
	else if (fIn) {
		be_app->PostMessage(eToolTipStop);
		fIn = false;
	}
	BButton::MouseMoved(where, code, msg);
}


//====================================================================
//
// BCheckBox sub-class that adds a ToolTip
//
//====================================================================

TToolTipCheckBox::TToolTipCheckBox(BRect rect,
								   const char *name,
								   const char *label,
								   BMessage *msg,
								   const char *tip,
								   uint32 resize_mask,
								   uint32 flags)
				 :BCheckBox(rect, name, label, msg, resize_mask, flags),
				  fIn(false)
{
	// cache the ToolTip string
	fTip = (char *)malloc(strlen(tip) + 1);
	strcpy(fTip, tip);
}

//--------------------------------------------------------------------

TToolTipCheckBox::~TToolTipCheckBox()
{
	// kill off any current ToolTip
	if (fIn)
		be_app->PostMessage(eToolTipStop);
	// free cached ToolTip
	free(fTip);
}

//--------------------------------------------------------------------

void TToolTipCheckBox::MouseMoved(BPoint where, uint32 code, const BMessage *msg)
{
	// if mouse has moved into our view, our window is active and it previously
	// wasn't in, send a message to start the ToolTip
	if ((Bounds().Contains(where)) && (Window()->IsActive())) {
		if (!fIn) {
			BMessage	msg(eToolTipStart);

			msg.AddPoint("start", ConvertToScreen(where));
			msg.AddRect("bounds", ConvertToScreen(Bounds()));
			msg.AddString("string", fTip);
			be_app->PostMessage(&msg);
			fIn = true;
		}
	}
	// otherwise stop the message
	else if (fIn) {
		be_app->PostMessage(eToolTipStop);
		fIn = false;
	}
	BCheckBox::MouseMoved(where, code, msg);
}


//====================================================================
//
// BColorControl sub-class that adds a ToolTip
//
//====================================================================

TToolTipColorControl::TToolTipColorControl(BPoint start,
										   color_control_layout layout,
										   float cell_size,
										   const char *name,
										   const char *tip,
										   BMessage *message,
										   bool use_offscreen)
					 :BColorControl(start, layout, cell_size, name, message, use_offscreen),
					  fIn(false)
{
	// cache the ToolTip string
	fTip = (char *)malloc(strlen(tip) + 1);
	strcpy(fTip, tip);
}

//--------------------------------------------------------------------

TToolTipColorControl::~TToolTipColorControl()
{
	// kill off any current ToolTip
	if (fIn)
		be_app->PostMessage(eToolTipStop);
	// free cached ToolTip
	free(fTip);
}

//--------------------------------------------------------------------

void TToolTipColorControl::MouseMoved(BPoint where, uint32 code, const BMessage *msg)
{
	// if mouse has moved into our view, our window is active and it previously
	// wasn't in, send a message to start the ToolTip
	if ((Bounds().Contains(where)) && (Window()->IsActive())) {
		if (!fIn) {
			BMessage	msg(eToolTipStart);

			msg.AddPoint("start", ConvertToScreen(where));
			msg.AddRect("bounds", ConvertToScreen(Bounds()));
			msg.AddString("string", fTip);
			be_app->PostMessage(&msg);
			fIn = true;
		}
	}
	// otherwise stop the message
	else if (fIn) {
		be_app->PostMessage(eToolTipStop);
		fIn = false;
	}
	BColorControl::MouseMoved(where, code, msg);
}


//====================================================================
//
// BRadioButton sub-class that adds a ToolTip
//
//====================================================================

TToolTipRadioButton::TToolTipRadioButton(BRect rect,
										 const char *name,
										 const char *label,
										 BMessage *msg,
										 const char *tip,
										 uint32 resize_mask,
										 uint32 flags)
					:BRadioButton(rect, name, label, msg, resize_mask, flags),
					 fIn(false)
{
	// cache the ToolTip string
	fTip = (char *)malloc(strlen(tip) + 1);
	strcpy(fTip, tip);
}

//--------------------------------------------------------------------

TToolTipRadioButton::~TToolTipRadioButton()
{
	// kill off any current ToolTip
	if (fIn)
		be_app->PostMessage(eToolTipStop);
	// free cached ToolTip
	free(fTip);
}

//--------------------------------------------------------------------

void TToolTipRadioButton::MouseMoved(BPoint where, uint32 code, const BMessage *msg)
{
	// if mouse has moved into our view, our window is active and it previously
	// wasn't in, send a message to start the ToolTip
	if ((Bounds().Contains(where)) && (Window()->IsActive())) {
		if (!fIn) {
			BMessage	msg(eToolTipStart);

			msg.AddPoint("start", ConvertToScreen(where));
			msg.AddRect("bounds", ConvertToScreen(Bounds()));
			msg.AddString("string", fTip);
			be_app->PostMessage(&msg);
			fIn = true;
		}
	}
	// otherwise stop the message
	else if (fIn) {
		be_app->PostMessage(eToolTipStop);
		fIn = false;
	}
	BRadioButton::MouseMoved(where, code, msg);
}


//====================================================================
//
// BPictureButtonButton sub-class that adds a ToolTip
//
//====================================================================

TToolTipPictureButton::TToolTipPictureButton(BRect rect,
											 const char *name,
											 BPicture *off,
											 BPicture *on,
											 BMessage *msg,
											 const char *tip,
											 uint32 behavior,
											 uint32 resize_mask,
											 uint32 flags)
					:BPictureButton(rect, name, off, on, msg, behavior, resize_mask, flags),
					 fIn(false)
{
	// cache the ToolTip string
	fTip = (char *)malloc(strlen(tip) + 1);
	strcpy(fTip, tip);
}

//--------------------------------------------------------------------

TToolTipPictureButton::~TToolTipPictureButton()
{
	// kill off any current ToolTip
	if (fIn)
		be_app->PostMessage(eToolTipStop);
	// free cached ToolTip
	free(fTip);
}

//--------------------------------------------------------------------

void TToolTipPictureButton::MouseMoved(BPoint where, uint32 code, const BMessage *msg)
{
	// if mouse has moved into our view, our window is active and it previously
	// wasn't in, send a message to start the ToolTip
	if ((Bounds().Contains(where)) && (Window()->IsActive())) {
		if (!fIn) {
			BMessage	msg(eToolTipStart);

			msg.AddPoint("start", ConvertToScreen(where));
			msg.AddRect("bounds", ConvertToScreen(Bounds()));
			msg.AddString("string", fTip);
			be_app->PostMessage(&msg);
			fIn = true;
		}
	}
	// otherwise stop the message
	else if (fIn) {
		be_app->PostMessage(eToolTipStop);
		fIn = false;
	}
	BPictureButton::MouseMoved(where, code, msg);
}


//====================================================================
//
// BSlider sub-class that adds a ToolTip
//
//====================================================================

TToolTipSlider::TToolTipSlider(BRect rect,
							   const char *name,
							   const char *label,
							   BMessage *msg,
							   int32 min_value,
							   int32 max_value,
							   const char *tip,
							   thumb_style thumb,
							   uint32 resize_mask,
							   uint32 flags)
			   :BSlider(rect, name, label, msg, min_value, max_value, thumb, resize_mask, flags),
			    fIn(false)
{
	// cache the ToolTip string
	fTip = (char *)malloc(strlen(tip) + 1);
	strcpy(fTip, tip);
}

//--------------------------------------------------------------------

TToolTipSlider::~TToolTipSlider()
{
	// kill off any current ToolTip
	if (fIn)
		be_app->PostMessage(eToolTipStop);
	// free cached ToolTip
	free(fTip);
}

//--------------------------------------------------------------------

void TToolTipSlider::MouseMoved(BPoint where, uint32 code, const BMessage *msg)
{
	// if mouse has moved into our view, our window is active and it previously
	// wasn't in, send a message to start the ToolTip
	if ((Bounds().Contains(where)) && (Window()->IsActive())) {
		if (!fIn) {
			BMessage	msg(eToolTipStart);

			msg.AddPoint("start", ConvertToScreen(where));
			msg.AddRect("bounds", ConvertToScreen(Bounds()));
			msg.AddString("string", fTip);
			be_app->PostMessage(&msg);
			fIn = true;
		}
	}
	// otherwise stop the message
	else if (fIn) {
		be_app->PostMessage(eToolTipStop);
		fIn = false;
	}
	BSlider::MouseMoved(where, code, msg);
}


//====================================================================
//
// BTextControl sub-class that adds a ToolTip
//
//====================================================================

TToolTipTextControl::TToolTipTextControl(BRect rect,
										 const char *name,
										 const char *label,
										 const char *text,
										 BMessage *msg,
										 const char * /*tip*/,
										 uint32 resize_mask,
										 uint32 flags)
					:BTextControl(rect, name, label, text, msg, resize_mask, flags),
					 fIn(false)
{
	SetDivider(StringWidth(label) + 4);
}

//--------------------------------------------------------------------

TToolTipTextControl::~TToolTipTextControl()
{
	// kill off any current ToolTip
	if (fIn)
		be_app->PostMessage(eToolTipStop);
}

//--------------------------------------------------------------------

void TToolTipTextControl::KeyDown(const char *bytes, int32 numBytes)
{
	if (fIn) {
		be_app->PostMessage(eToolTipStop);
		fIn = false;
	}
	BTextControl::KeyDown(bytes, numBytes);
}

//--------------------------------------------------------------------

void TToolTipTextControl::MouseMoved(BPoint where, uint32 code, const BMessage *msg)
{
	// if mouse has moved into our view, our window is active and it previously
	// wasn't in, send a message to start the ToolTip
	if ((Bounds().Contains(where)) && (Window()->IsActive())) {
		if (!fIn) {
			BMessage	msg(eToolTipStart);

			msg.AddPoint("start", ConvertToScreen(where));
			msg.AddRect("bounds", ConvertToScreen(Bounds()));
			(strlen(Text())) ?
				msg.AddString("string", Text()) :
				msg.AddString("string", "This space for rent");
			be_app->PostMessage(&msg);
			fIn = true;
		}
	}
	// otherwise stop the message
	else if (fIn) {
		be_app->PostMessage(eToolTipStop);
		fIn = false;
	}
	BTextControl::MouseMoved(where, code, msg);
}
